######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

require 'openstudio'

require 'openstudio/ruleset/ShowRunnerOutput'

require "#{File.dirname(__FILE__)}/../measure.rb"

require 'test/unit'

class ReduceElectricEquipmentLoadsByPercentage_Test < Test::Unit::TestCase

  def test_DefaultArgs

    # create an instance of the measure
    measure = ReduceElectricEquipmentLoadsByPercentage.new

    # create an instance of a runner
    runner = OpenStudio::Ruleset::OSRunner.new

    # load the test model
    translator = OpenStudio::OSVersion::VersionTranslator.new
    path = OpenStudio::Path.new(File.dirname(__FILE__) + "/EnvelopeAndLoadTestModel_01.osm")
    model = translator.loadModel(path)
    assert((not model.empty?))
    model = model.get

    # refresh arguments
    arguments = measure.arguments(model)

    # fill in argument_map
    argument_map = OpenStudio::Ruleset::OSArgumentMap.new

    space_type = arguments[0].clone
    argument_map["space_type"] = space_type

    reduction_percent = arguments[1].clone
    argument_map["reduction_percent"] = reduction_percent

    count = 1

    material_and_installation_cost = arguments[count += 1].clone
    argument_map["material_and_installation_cost"] = material_and_installation_cost

    demolition_cost = arguments[count += 1].clone
    argument_map["demolition_cost"] = demolition_cost

    years_until_costs_start = arguments[count += 1].clone
    argument_map["years_until_costs_start"] = years_until_costs_start

    initial_demo_costs = arguments[count += 1].clone
    argument_map["initial_demo_costs"] = initial_demo_costs

    expected_life = arguments[count += 1].clone
    argument_map["expected_life"] = expected_life

    om_cost = arguments[count += 1].clone
    argument_map["om_cost"] = om_cost

    om_frequency = arguments[count += 1].clone
    argument_map["om_frequency"] = om_frequency

    # test model before measure
    equip_def1_found = false
    equip_def2_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)

    measure.run(model, runner, argument_map)
    result = runner.result
    assert(result.value.valueName == "Success", show_output(result))

    # test model after measure
    equip_def1_found = false
    equip_def2_found = false
    equip_def1_clone_found = false
    equip_def2_clone_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        # original not changed
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        # original not changed
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif /100W Thing .* percent reduction/.match(equip_def.name.get)
        equip_def1_clone_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(95.0, equip_def.designLevel.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      elsif /ASHRAE_189\.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef .* percent reduction/.match(equip_def.name.get)
        equip_def2_clone_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        #assert_in_delta(4.026, equip_def.wattsperSpaceFloorArea.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)
    #assert(equip_def1_clone_found)
    assert(equip_def2_clone_found)

    # test messages
    assert((not result.finalCondition.empty?))
    #assert(/EPD was reduced by 95\.00 in 0\.00 ft\^2 of the building/.match(result.finalCondition.get.logMessage))

    # loop over info warnings

    # loop over warnings
    expected_messages = Hash.new
    #expected_messages["The requested wall insulation R-value of 50.0 ft^2*h*R/Btu is abnormally high"] = false
    expected_messages["Construction 'Test_No Insulation' does not appear to have an insulation layer and was not altered."] = false
    result.warnings.each do |warning|
      expected_messages.each_key do |message|
        if Regexp.new(message).match(warning.logMessage)
          assert(expected_messages[message] == false, "Message '#{message}' found multiple times")
          expected_messages[message] = true
        end
      end
    end
    #expected_messages.each_pair do |message, found|
    #  assert(found, "Message '#{message}' not found")
    #end
  end

  #################################################################################################
  #################################################################################################
  
  def test_NoReductionNoCost

    # create an instance of the measure
    measure = ReduceElectricEquipmentLoadsByPercentage.new

    # create an instance of a runner
    runner = OpenStudio::Ruleset::OSRunner.new

    # load the test model
    translator = OpenStudio::OSVersion::VersionTranslator.new
    path = OpenStudio::Path.new(File.dirname(__FILE__) + "/EnvelopeAndLoadTestModel_01.osm")
    model = translator.loadModel(path)
    assert((not model.empty?))
    model = model.get

    # refresh arguments
    arguments = measure.arguments(model)

    # fill in argument_map
    argument_map = OpenStudio::Ruleset::OSArgumentMap.new

    space_type = arguments[0].clone
    argument_map["space_type"] = space_type

    reduction_percent = arguments[1].clone
    assert(reduction_percent.setValue(0))
    argument_map["reduction_percent"] = reduction_percent

    count = 1

    material_and_installation_cost = arguments[count += 1].clone
    assert(material_and_installation_cost.setValue(0))
    argument_map["material_and_installation_cost"] = material_and_installation_cost

    demolition_cost = arguments[count += 1].clone
    argument_map["demolition_cost"] = demolition_cost

    years_until_costs_start = arguments[count += 1].clone
    argument_map["years_until_costs_start"] = years_until_costs_start

    initial_demo_costs = arguments[count += 1].clone
    argument_map["initial_demo_costs"] = initial_demo_costs

    expected_life = arguments[count += 1].clone
    argument_map["expected_life"] = expected_life

    om_cost = arguments[count += 1].clone
    argument_map["om_cost"] = om_cost

    om_frequency = arguments[count += 1].clone
    argument_map["om_frequency"] = om_frequency

     # test model before measure
    equip_def1_found = false
    equip_def2_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)

    measure.run(model, runner, argument_map)
    result = runner.result
    assert(result.value.valueName == "NA", show_output(result))

    # test model after measure
    equip_def1_found = false
    equip_def2_found = false
    equip_def1_clone_found = false
    equip_def2_clone_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        # original not changed
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        # original not changed
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif /100W Thing .* percent reduction/.match(equip_def.name.get)
        equip_def1_clone_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(95.0, equip_def.designLevel.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      elsif /ASHRAE_189\.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef .* percent reduction/.match(equip_def.name.get)
        equip_def2_clone_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        #assert_in_delta(4.026, equip_def.wattsperSpaceFloorArea.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)
    #assert(equip_def1_clone_found)
    assert(equip_def2_clone_found)

    # test messages
    assert((not result.finalCondition.empty?))
    #assert(/EPD was reduced by 95\.00 in 0\.00 ft\^2 of the building/.match(result.finalCondition.get.logMessage))

    # loop over info warnings

    # loop over warnings
    expected_messages = Hash.new
    #expected_messages["The requested wall insulation R-value of 50.0 ft^2*h*R/Btu is abnormally high"] = false
    expected_messages["Construction 'Test_No Insulation' does not appear to have an insulation layer and was not altered."] = false
    result.warnings.each do |warning|
      expected_messages.each_key do |message|
        if Regexp.new(message).match(warning.logMessage)
          assert(expected_messages[message] == false, "Message '#{message}' found multiple times")
          expected_messages[message] = true
        end
      end
    end
    #expected_messages.each_pair do |message, found|
    #  assert(found, "Message '#{message}' not found")
    #end
  end

  #################################################################################################
  #################################################################################################

  def test_ReduceElectricEquipmentLoadsByPercentage_RetrofitUncostedBuilding

    # create an instance of the measure
    measure = ReduceElectricEquipmentLoadsByPercentage.new

    # create an instance of a runner
    runner = OpenStudio::Ruleset::OSRunner.new

    # load the test model
    translator = OpenStudio::OSVersion::VersionTranslator.new
    path = OpenStudio::Path.new(File.dirname(__FILE__) + "/EnvelopeAndLoadTestModel_01.osm")
    model = translator.loadModel(path)
    assert((not model.empty?))
    model = model.get

    # refresh arguments
    arguments = measure.arguments(model)

    # fill in argument_map
    argument_map = OpenStudio::Ruleset::OSArgumentMap.new

    space_type = arguments[0].clone
    assert(space_type.setValue("*Entire Building*"))
    argument_map["space_type"] = space_type

    reduction_percent = arguments[1].clone
    assert(reduction_percent.setValue(45.0))
    argument_map["reduction_percent"] = reduction_percent

    count = 1

    material_and_installation_cost = arguments[count += 1].clone
    #puts material_and_installation_cost.displayName
    assert(material_and_installation_cost.setValue(50.0))
    argument_map["material_and_installation_cost"] = material_and_installation_cost

    demolition_cost = arguments[count += 1].clone
    #puts demolition_cost.displayName
    assert(demolition_cost.setValue(18.0))
    argument_map["demolition_cost"] = demolition_cost

    years_until_costs_start = arguments[count += 1].clone
    #puts years_until_costs_start.displayName
    assert(years_until_costs_start.setValue(1))
    argument_map["years_until_costs_start"] = years_until_costs_start

    initial_demo_costs = arguments[count += 1].clone
    #puts initial_demo_costs.displayName
    assert(initial_demo_costs.setValue(true))
    argument_map["initial_demo_costs"] = initial_demo_costs

    expected_life = arguments[count += 1].clone
    #puts expected_life.displayName
    assert(expected_life.setValue(20))
    argument_map["expected_life"] = expected_life

    om_cost = arguments[count += 1].clone
    #puts om_cost.displayName
    assert(om_cost.setValue(20))
    argument_map["om_cost"] = om_cost

    om_frequency = arguments[count += 1].clone
    #puts om_frequency.displayName
    assert(om_frequency.setValue(20))
    argument_map["om_frequency"] = om_frequency

    # test model before measure
    equip_def1_found = false
    equip_def2_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)

    measure.run(model, runner, argument_map)
    result = runner.result
    assert(result.value.valueName == "Success", show_output(result))

    # test model after measure
    equip_def1_found = false
    equip_def2_found = false
    equip_def1_clone_found = false
    equip_def2_clone_found = false
    model.getElectricEquipmentDefinitions.each do |equip_def|
      if equip_def.name.get == "100W Thing"
        # original not changed
        equip_def1_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(100.0, equip_def.designLevel.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif equip_def.name.get == "ASHRAE_189.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef"
        # original not changed
        equip_def2_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        assert_in_delta(7.32, equip_def.wattsperSpaceFloorArea.get, 0.01)
        assert(equip_def.lifeCycleCosts.empty?)
      elsif /100W Thing .* percent reduction/.match(equip_def.name.get)
        equip_def1_clone_found = true
        assert((not equip_def.designLevel.empty?))
        assert_in_delta(95.0, equip_def.designLevel.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      elsif /ASHRAE_189\.1-2009_ClimateZone 4-8_LargeHotel_GuestRoom_ElecDef .* percent reduction/.match(equip_def.name.get)
        equip_def2_clone_found = true
        assert((not equip_def.wattsperSpaceFloorArea.empty?))
        #assert_in_delta(4.026, equip_def.wattsperSpaceFloorArea.get, 0.01)
        #assert_equal(1, equip_def.lifeCycleCosts.size)
        #assert_equal(15, equip_def.lifeCycleCosts[0].materialCost)
      end
    end
    assert(equip_def1_found)
    assert(equip_def2_found)
    #assert(equip_def1_clone_found)
    assert(equip_def2_clone_found)

    # test messages
    assert((not result.finalCondition.empty?))
    #assert(/EPD was reduced by 95\.00 in 0\.00 ft\^2 of the building/.match(result.finalCondition.get.logMessage))

    # loop over info warnings

    # loop over warnings
    expected_messages = Hash.new
    #expected_messages["The requested wall insulation R-value of 50.0 ft^2*h*R/Btu is abnormally high"] = false
    expected_messages["Construction 'Test_No Insulation' does not appear to have an insulation layer and was not altered."] = false
    result.warnings.each do |warning|
      expected_messages.each_key do |message|
        if Regexp.new(message).match(warning.logMessage)
          assert(expected_messages[message] == false, "Message '#{message}' found multiple times")
          expected_messages[message] = true
        end
      end
    end
    #expected_messages.each_pair do |message, found|
    #  assert(found, "Message '#{message}' not found")
    #end
  end

end
